@@ -439,3 +439,75 @@ class TestTimestreamLoader_UnitTests:
         dt = datetime.combine(date(2021, 1, 1), datetime.min.time()).replace(tzinfo=pytz.UTC)
         assert loader._TimestreamLoader__dt_to_ms(dt) == 1609459200000
         assert loader._TimestreamLoader__dt_to_ms(None) == -1
+
+    @pytest.mark.unit
+    def test_does_database_exist(self):
+        # Set expectations
+        expected_error = ClientError({'response': {'Error': {'Message': 'The database default does not exist.', 'Code': 'ResourceNotFoundException'}, 'ResponseMetadata': {'RequestId': 'V6E3FBONKV7C6YK3WLSVISIJGI', 'HTTPStatusCode': 404, 'HTTPHeaders': {'x-amzn-requestid': 'V6E3FBONKV7C6YK3WLSVISIJGI', 'content-type': 'application/x-amz-json-1.0', 'content-length': '129', 'date': 'Wed, 20 Oct 2021 17:04:42 GMT'}, 'RetryAttempts': 0}, 'Message': 'The database default does not exist.'}, 'operation_name': 'DescribeDatabase'}, 'DescribeDatabase')
+        mock_response = {'Database': {'Arn': 'arn:aws:timestream:us-east-1:000000000000:database/test', 'DatabaseName': 'test', 'TableCount': 1, 'KmsKeyId': 'arn:aws:kms:us-east-1:000000000000:key/12345678-1234-1234-1234-1234567890ab', 'CreationTime': datetime(2021, 7, 19, 16, 25, 26, 419000), 'LastUpdatedTime': datetime(2021, 10, 8, 16, 25, 37, 893000)}, 'ResponseMetadata': {'RequestId': '1234567890abcdefghijklmnop', 'HTTPStatusCode': 200, 'HTTPHeaders': {'x-amzn-requestid': '1234567890abcdefghijklmnop', 'content-type': 'application/x-amz-json-1.0', 'content-length': '289', 'date': 'Wed, 20 Oct 2021 16:54:48 GMT'}, 'RetryAttempts': 0}}
+
+        # Set up mocks
+        mock_client = MockApi()
+        mock_client.on(
+            'describe_database',
+            DatabaseName='default',
+        ).raise_exception(expected_error)
+        mock_client.on(
+            'describe_database',
+            DatabaseName='test',
+        ).return_val(mock_response)
+        mock_session = MockApi()
+        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(mock_client)
+
+        # Test False when using TimestreamLoader's database attribtue
+        loader = TimestreamLoader('default', 'default', mock_session)
+        result = loader.does_database_exist()
+        assert not result
+
+        # Test exception raised if not found
+        with pytest.raises(ClientError) as e_info:
+            loader.does_database_exist(raise_err=True)
+
+        # Test True using test name
+        result = loader.does_database_exist('test')
+        assert result
+
+        mock_session.assert_expectations()
+        mock_client.assert_expectations()
+
+    @pytest.mark.unit
+    def test_does_table_exist(self):
+        # Set expectations
+        expected_error = ClientError({'response': {'Error': {'Message': 'The table default does not exist.', 'Code': 'ResourceNotFoundException'}, 'ResponseMetadata': {'RequestId': 'V6E3FBONKV7C6YK3WLSVISIJGI', 'HTTPStatusCode': 404, 'HTTPHeaders': {'x-amzn-requestid': 'V6E3FBONKV7C6YK3WLSVISIJGI', 'content-type': 'application/x-amz-json-1.0', 'content-length': '129', 'date': 'Wed, 20 Oct 2021 17:04:42 GMT'}, 'RetryAttempts': 0}, 'Message': 'The table default does not exist.'}, 'operation_name': 'DescribeTable'}, 'DescribeTable')
+        mock_response = {'Table': {'Arn': 'arn:aws:timestream:us-east-1:000000000000:database/test/table/test', 'TableName': 'test', 'DatabaseName': 'test', 'TableStatus': 'ACTIVE', 'RetentionProperties': {'MemoryStoreRetentionPeriodInHours': 720, 'MagneticStoreRetentionPeriodInDays': 1}, 'CreationTime': datetime(2021, 8, 19, 18, 21, 1, 519000), 'LastUpdatedTime': datetime(2021, 8, 31, 12, 36, 4, 503000)}, 'ResponseMetadata': {'RequestId': 'SDKOJJS7STGCQXOFVLXPBYDRKI', 'HTTPStatusCode': 200, 'HTTPHeaders': {'x-amzn-requestid': 'SDKOJJS7STGCQXOFVLXPBYDRKI', 'content-type': 'application/x-amz-json-1.0', 'content-length': '366', 'date': 'Wed, 20 Oct 2021 17:41:09 GMT'}, 'RetryAttempts': 1}}
+
+        # Set up mocks
+        mock_client = MockApi()
+        mock_client.on(
+            'describe_table',
+            DatabaseName='default',
+            TableName='default'
+        ).raise_exception(expected_error)
+        mock_client.on(
+            'describe_table',
+            DatabaseName='test',
+            TableName='test'
+        ).return_val(mock_response)
+        mock_session = MockApi()
+        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(mock_client)
+
+        # Test False when using TimestreamLoader's database and table attribtue
+        loader = TimestreamLoader('default', 'default', mock_session)
+        result = loader.does_table_exist()
+        assert not result
+
+        # Test exception raised if not found
+        with pytest.raises(ClientError) as e_info:
+            loader.does_table_exist(raise_err=True)
+
+        # Test True using test name
+        result = loader.does_table_exist('test', 'test')
+        assert result
+
+        mock_session.assert_expectations()
+        mock_client.assert_expectations()
