@@ -6,6 +6,7 @@ from botocore.config import Config
 from botocore.exceptions import ClientError
 from datetime import datetime
 from timestream import DEFAULT_LOGGER, log_it
+from timestream.batch import batch_records, DEFAULT_MAX_BATCH_SIZE, DEFAULT_MIN_COMMON_DIMENSIONS
 from timestream.dimensions import TimestreamDimensions
 from timestream.load_result import TimestreamLoadResult
 from timestream.record import RejectedTimestreamRecord, TimestreamRecord
@@ -43,37 +44,138 @@ class TimestreamLoader:
             self.__current_time_ms = current_time_ms
 
     @log_it
+    def does_database_exist(
+        self,
+        database_name: str = None,
+        raise_err: bool = False
+    ) -> bool:
+        '''
+        Performs a boto3 describe_database API call and returns True if the
+        specified database_name exists
+
+        Parameters
+        ----------
+        database_name : str
+            The name of the database you want to check, or self.database_name
+
+        raise_err : bool, Default : False
+            Whether to raise the boto3 ResourceNotFoundException or just return
+            False
+
+        Returns
+        -------
+        bool
+
+        Raises
+        ------
+        botocore.exceptions.ClientError/ResourceNotFoundException
+        '''
+        db_name = database_name if database_name else self.__database_name
+        try:
+            response = self.__timestream_write_client.describe_database(
+                DatabaseName=db_name)
+            result = True
+        except ClientError as ex:
+            if raise_err:
+                raise ex
+            else:
+                result = False
+        return result
+
+    @log_it
+    def does_table_exist(
+        self,
+        database_name: str = None,
+        table_name: str = None,
+        raise_err: bool = False
+    ) -> bool:
+        '''
+        Performs a boto3 describe_table API call and returns True if the
+        specified table_name exists in the database
+
+        Parameters
+        ----------
+        database_name : str
+            The name of the database you want to check, or self.__database_name
+
+        table_name : str
+            The name of the database you want to check, or self.__table_name
+
+        raise_err : bool, Default : False
+            Whether to raise the boto3 ResourceNotFoundException or just return
+            False
+
+        Returns
+        -------
+        bool
+
+        Raises
+        ------
+        botocore.exceptions.ClientError/ResourceNotFoundException
+        '''
+        db_name = database_name if database_name else self.__database_name
+        table = table_name if table_name else self.__table_name
+        try:
+            response = self.__timestream_write_client.describe_table(
+                DatabaseName=db_name,
+                TableName=table)
+            result = True
+        except ClientError as ex:
+            if raise_err:
+                raise ex
+            else:
+                result = False
+        return result
+
     def load(
         self,
         data: list,
         capture_time: datetime,
         source: str = None,
         upsert: bool = False,
-        ignore_parse_error: bool = True
+        ignore_parse_error: bool = True,
+        max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
+        min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS
     ) -> dict:
         '''
         Performs a Timestream load. Validates the input and parses the data into Timestream records.
         Returns the result of the load after writing to Timestream.
         '''
-        return self.__load(data, capture_time, source, upsert, ignore_parse_error, False).to_dict()
+        return self.__load(
+            data,
+            capture_time,
+            source, upsert,
+            ignore_parse_error,
+            max_batch_size,
+            min_common_dimensions,
+            False,
+        ).to_dict()
 
-    @log_it
     def load_preview(
         self,
         data: list,
         capture_time: datetime,
         source: str = None,
         upsert: bool = False,
-        ignore_parse_error: bool = True
+        ignore_parse_error: bool = True,
+        max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
+        min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS
     ) -> dict:
         '''
         Performs a dry run of a Timestream load, including validating the input values and parsing data
         into Timestream records.
         Returns the result of the load before writing to Timestream.
         '''
-        return self.__load(data, capture_time, source, upsert, ignore_parse_error, True).to_dict()
+        return self.__load(
+            data,
+            capture_time,
+            source, upsert,
+            ignore_parse_error,
+            max_batch_size,
+            min_common_dimensions,
+            True,
+        ).to_dict()
 
-    @log_it
     def __load(
         self,
         data: list,
@@ -81,35 +183,36 @@ class TimestreamLoader:
         source: str = None,
         upsert: bool = False,
         ignore_parse_error: bool = True,
+        max_batch_size: int = DEFAULT_MAX_BATCH_SIZE,
+        min_common_dimensions: int = DEFAULT_MIN_COMMON_DIMENSIONS,
         preview: bool = False
     ) -> TimestreamLoadResult:
+        self._logger.info(f'Loading data with {len(data)} elements')
+
         # Parse incoming data into TimestreamRecords
         result = self.__parse_data(data, upsert, ignore_parse_error)
         if preview or result.error:
             return result
 
-        # Group records by common dimensions using a hash string
-        grouped_records, common_dimensions = {}, {}
-        for r in result.accepted_records:
-            hashstr = r.dimensions.hashstr()
-            common_dimensions[hashstr] = r.dimensions
-            grouped_records[hashstr] = grouped_records.get(hashstr) or []
-            grouped_records[hashstr].append(r)
-
-        # Assert all groups have records
-        assert all(len(records) > 0 for records in grouped_records.values()), "Not all groups have records (shouldn't be possible)"
+        # Batch the accepted records by common dimensions
+        batches = batch_records(
+            result.accepted_records,
+            max_batch_size=max_batch_size,
+            min_common_dimensions=min_common_dimensions
+        )
+        assert all(len(b.records) > 0 for b in batches), "Not all batches have records (shouldn't be possible)"
 
         # Write records in batches to Timestream
         result = TimestreamLoadResult()
-        for hashstr, records in grouped_records.items():
-            dimensions = common_dimensions[hashstr]
+        for b in batches:
+            dimensions = b.common_dimensions
             if source:
                 dimensions.add_dimension('source', source, overwrite=True)
-            if records and upsert:
-                accepted_records, rejected_records = self.__write_upsert_records(dimensions, records)
-            elif records:
-                accepted_records, rejected_records = self.__write_insert_records(dimensions, records, capture_time)
-            
+            if b.records and upsert:
+                accepted_records, rejected_records = self.__write_upsert_records(dimensions, b.records)
+            elif b.records:
+                accepted_records, rejected_records = self.__write_insert_records(dimensions, b.records, capture_time)
+
             if accepted_records:
                 result.add_accepted_records(accepted_records)
             if rejected_records:
@@ -117,13 +220,13 @@ class TimestreamLoader:
                 result.error = Exception('Some records were rejected by Timestream')
         return result
 
-    @log_it
     def __write_insert_records(
         self,
-        dimensions: TimestreamDimensions,
+        common_dimensions: TimestreamDimensions,
         records: List[TimestreamRecord],
         capture_time: datetime
     ) -> tuple:
+        self._logger.info(f'Writing insert records with {len(records)} records')
         assert capture_time, 'A capture time needs to be provided'
         capture_time_ms = self.__dt_to_ms(capture_time)
         accepted_records, rejected_records = records, []
@@ -132,42 +235,47 @@ class TimestreamLoader:
                 DatabaseName=self.__database_name,
                 TableName=self.__table_name,
                 CommonAttributes={
-                    'Dimensions': dimensions.loadable(),
+                    'Dimensions': common_dimensions.loadable(),
                     'MeasureValueType': 'DOUBLE',
                     'Time': str(capture_time_ms),
                     'Version': 1
                 },
-                Records=[r.loadable() for r in records]
+                Records=[
+                    r.loadable(omit_dimensions=common_dimensions)
+                    for r in records
+                ]
             )
         except ClientError as e:
             self._logger.error(f'Failed to write insert records: {e}')
             accepted_records, rejected_records = self.__process_client_error(e, records)
         return accepted_records, rejected_records
 
-    @log_it
     def __write_upsert_records(
         self,
-        dimensions: TimestreamDimensions,
+        common_dimensions: TimestreamDimensions,
         records: List[TimestreamRecord]
     ) -> tuple:
+        self._logger.info(f'Writing upsert records with {len(records)} records')
         accepted_records, rejected_records = records, []
         try:
             _ = self.__timestream_write_client.write_records(
                 DatabaseName=self.__database_name,
                 TableName=self.__table_name,
                 CommonAttributes={
-                    'Dimensions': dimensions.loadable(),
+                    'Dimensions': common_dimensions.loadable(),
                     'MeasureValueType': 'DOUBLE',
                     'Version': self.__current_time_ms
                 },
-                Records=[r.loadable() for r in records]
+                Records=[
+                    r.loadable(omit_dimensions=common_dimensions)
+                    for r in records
+                ]
             )
         except ClientError as e:
             self._logger.error(f'Failed to write upsert records: {e}')
             accepted_records, rejected_records = self.__process_client_error(e, records)
         return accepted_records, rejected_records
 
-    @log_it
     def __process_client_error(self, e: ClientError, records: List[TimestreamRecord]) -> tuple:
         '''
         Processes a boto ClientError from a Timestream write operation. Logs error messages
@@ -182,16 +290,15 @@ class TimestreamLoader:
                 error=Exception(rejection_data['Reason'])
             )
             self._logger.error(f'Rejected record:\n{rejected_record}')
-            rejected_records.append(rejected_record) 
+            rejected_records.append(rejected_record)
             rejected_indices.append(rejection_data['RecordIndex'])
-        
+
         accepted_records = [
             records[i] for i in range(len(records))
             if i not in rejected_indices
         ]
         return accepted_records, rejected_records
 
-    @log_it
     def __parse_data(self, data: List[dict], upsert: bool, ignore_parse_error: bool) -> TimestreamLoadResult:
         '''
         Parses the given data and returns a TimestreamLoadResult that contains:
@@ -213,7 +320,6 @@ class TimestreamLoader:
             result.add_accepted_records(records_from_item)
         return result
 
-    @log_it
     def __parse_item(self, item: dict, upsert: bool, ignore_parse_error: bool) -> List[TimestreamRecord]:
         '''
         Parses a single dictionary into Timestream records. A single dictionary may break down into
